package hu.bme.mit.books.graphiti.editor.features;

import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.IAddContext;
import org.eclipse.graphiti.features.impl.AbstractAddShapeFeature;
import org.eclipse.graphiti.mm.algorithms.Rectangle;
import org.eclipse.graphiti.mm.algorithms.Text;
import org.eclipse.graphiti.mm.algorithms.styles.Orientation;
import org.eclipse.graphiti.mm.pictograms.BoxRelativeAnchor;
import org.eclipse.graphiti.mm.pictograms.ContainerShape;
import org.eclipse.graphiti.mm.pictograms.Diagram;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.mm.pictograms.Shape;
import org.eclipse.graphiti.services.Graphiti;
import org.eclipse.graphiti.services.IGaService;
import org.eclipse.graphiti.services.IPeCreateService;
import org.eclipse.graphiti.util.IColorConstant;

import books.Author;

public class AddAuthorFeature extends AbstractAddShapeFeature {

	private static final IColorConstant E_CLASS_TEXT_FOREGROUND = IColorConstant.BLACK;

	private static final IColorConstant E_CLASS_FOREGROUND = IColorConstant.BLUE;

	private static final IColorConstant E_CLASS_BACKGROUND = IColorConstant.YELLOW;

	public AddAuthorFeature(IFeatureProvider fp) {
		super(fp);
	}

	@Override
	public boolean canAdd(IAddContext context) {
		return (context.getNewObject() instanceof Author && context
				.getTargetContainer() instanceof Diagram);
	}

	@Override
	public PictogramElement add(IAddContext context) {
		Author addedAuthor = (Author) context.getNewObject();
		Diagram targetDiagram = (Diagram) context.getTargetContainer();

		// CONTAINER SHAPE WITH ROUNDED RECTANGLE
		IPeCreateService peCreateService = Graphiti.getPeCreateService();
		ContainerShape containerShape = peCreateService.createContainerShape(
				targetDiagram, true);

		// define a default size for the shape
		int width = 100;
		int height = 25;
		IGaService gaService = Graphiti.getGaService();
		Rectangle rectangle; // need to access it later

		{
			// create and set graphics algorithm
			rectangle = gaService.createRectangle(containerShape);
			rectangle.setForeground(manageColor(E_CLASS_FOREGROUND));
			rectangle.setBackground(manageColor(E_CLASS_BACKGROUND));
			rectangle.setLineWidth(2);
			gaService.setLocationAndSize(rectangle, context.getX(),
					context.getY(), width, height);

			// if added Class has no resource we add it to the resource
			// of the diagram
			// in a real scenario the business model would have its own resource
			if (addedAuthor.eResource() == null) {
				getDiagram().eResource().getContents().add(addedAuthor);
			}
			// create link and wire it
			link(containerShape, addedAuthor);
		}

		// SHAPE WITH TEXT
		{
			// create shape for text
			Shape shape = peCreateService.createShape(containerShape, false);

			// create and set text graphics algorithm
			Text text = gaService.createText(shape, addedAuthor.getName());
			text.setForeground(manageColor(E_CLASS_TEXT_FOREGROUND));
			text.setHorizontalAlignment(Orientation.ALIGNMENT_CENTER);
			// vertical alignment has as default value "center"
			text.setFont(gaService.manageDefaultFont(getDiagram(), false, true));
			gaService.setLocationAndSize(text, 0, 0, width, 20);

			// create link and wire it
			link(shape, addedAuthor);
		}

		// add a chopbox anchor to the shape
	      peCreateService.createChopboxAnchor(containerShape);
	 
	      // create an additional box relative anchor at middle-right
	      final BoxRelativeAnchor boxAnchor =
	           peCreateService.createBoxRelativeAnchor(containerShape);
	     
	      boxAnchor.setRelativeWidth(1.0);
	      boxAnchor.setRelativeHeight(0.38); //use golden section
	      boxAnchor.setReferencedGraphicsAlgorithm(rectangle);

	      // assign a rectangle graphics algorithm for the box relative anchor
	      // note, that the rectangle is inside the border of the rectangle shape
	      final Rectangle aRectangle = gaService.createPlainRectangle(boxAnchor);
	      aRectangle.setForeground(manageColor(E_CLASS_FOREGROUND));
	      aRectangle.setBackground(manageColor(E_CLASS_BACKGROUND));
	      aRectangle.setLineWidth(2);
	      gaService.setLocationAndSize(rectangle, -12, -6, 12, 12);
	     
	   // call the layout feature
	      layoutPictogramElement(containerShape);
		
		return containerShape;
	}

}
