package hu.bme.mit.books.graphiti.editor.features;

import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.IDirectEditingContext;
import org.eclipse.graphiti.features.impl.AbstractDirectEditingFeature;
import org.eclipse.graphiti.mm.algorithms.GraphicsAlgorithm;
import org.eclipse.graphiti.mm.algorithms.Text;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;

import books.Book;

public class BookDirectEditingFeature extends AbstractDirectEditingFeature {

	public BookDirectEditingFeature(IFeatureProvider fp) {
		super(fp);
	}

	@Override
	public int getEditingType() {
		return TYPE_TEXT;
	}

	@Override
	public boolean canDirectEdit(IDirectEditingContext context) {
        PictogramElement pe = context.getPictogramElement();
        Object bo = getBusinessObjectForPictogramElement(pe);
        GraphicsAlgorithm ga = context.getGraphicsAlgorithm();
        // support direct editing, if it is a EClass, and the user clicked
        // directly on the text and not somewhere else in the rectangle
        if (bo instanceof Book && ga instanceof Text) {
            return true;
        }
		return false;
	}

	@Override
	public String getInitialValue(IDirectEditingContext context) {
		Book book = (Book) getBusinessObjectForPictogramElement(context.getPictogramElement()); 
		return book.getTitle();
	}

	@Override
	public String checkValueValid(String value, IDirectEditingContext context) {
        if (value.length() < 1)
            return "Please enter a book title.";
        if (value.contains("\n"))
            return "Line breakes are not allowed in book titles.";
		return super.checkValueValid(value, context);
	}

	@Override
	public void setValue(String value, IDirectEditingContext context) {
		PictogramElement pe = context.getPictogramElement();
		Book book = (Book) getBusinessObjectForPictogramElement(pe);
		book.setTitle(value);
	}

}
