package hu.bme.mit.books.graphiti.editor.features;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.ICreateConnectionContext;
import org.eclipse.graphiti.features.context.impl.AddConnectionContext;
import org.eclipse.graphiti.features.impl.AbstractCreateConnectionFeature;
import org.eclipse.graphiti.mm.pictograms.Anchor;
import org.eclipse.graphiti.mm.pictograms.Connection;

import books.Author;
import books.Book;

public class CreateAuthorBookConnectionFeature extends AbstractCreateConnectionFeature {

	public CreateAuthorBookConnectionFeature(IFeatureProvider fp) {
		super(fp, "Connect Author", "Connects an Author to its books");
	}
	
	@Override
	public boolean canStartConnection(ICreateConnectionContext context) {
		EObject source = getEClass(context.getSourceAnchor());
		return (source instanceof Author);
	}

	@Override
	public boolean canCreate(ICreateConnectionContext context) {
		EObject source = getEClass(context.getSourceAnchor());
		EObject target = getEClass(context.getTargetAnchor());
		return (source instanceof Author && target instanceof Book &&
				!((Author)source).getBooks().contains(target));
	}

	@Override
	public Connection create(ICreateConnectionContext context) {
        Connection newConnection = null;
        
        // get EClasses which should be connected
        EObject source = getEClass(context.getSourceAnchor());
        EObject target = getEClass(context.getTargetAnchor());
 
        if (source instanceof Author && target instanceof Book) {
        	// create reference
        	Author author = (Author) source;
        	Book book = (Book) target;
        	author.getBooks().add(book);
            // add connection for business object
            AddConnectionContext addContext =
                new AddConnectionContext(context.getSourceAnchor(), context
                    .getTargetAnchor());
            newConnection =
                (Connection) getFeatureProvider().addIfPossible(addContext);
        }
       
        return newConnection;
	}


	/**
     * Returns the EObject belonging to the anchor, or null if not available.
     */
    private EObject getEClass(Anchor anchor) {
        if (anchor != null) {
            Object object =
                getBusinessObjectForPictogramElement(anchor.getParent());
            if (object instanceof EObject) {
                return (EObject) object;
            }
        }
        return null;
    }
}
