package hu.bme.mit.books.graphiti.editor.features;

import org.eclipse.graphiti.examples.common.ExampleUtil;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.ICreateContext;
import org.eclipse.graphiti.features.impl.AbstractCreateFeature;
import org.eclipse.graphiti.mm.pictograms.Diagram;

import books.Author;
import books.BooksFactory;

public class CreateAuthorFeature extends AbstractCreateFeature {

	public CreateAuthorFeature(IFeatureProvider fp) {
		super(fp, "Author", "Creates a new author object");
	}

	@Override
	public boolean canCreate(ICreateContext context) {
		return context.getTargetContainer() instanceof Diagram;
	}

	@Override
	public Object[] create(ICreateContext context) {
        // ask user for EClass name
        String newAuthorName = ExampleUtil.askString("Create Author", "Enter Author Name", "");
        if (newAuthorName == null || newAuthorName.trim().length() == 0) {
            return EMPTY;
        }
 
        // create EClass
        Author newAuthor = BooksFactory.eINSTANCE.createAuthor();
        // Add model element to resource.
        // We add the model element to the resource of the diagram for
        // simplicity's sake. Normally, a customer would use its own
        // model persistence layer for storing the business model separately.
        getDiagram().eResource().getContents().add(newAuthor);
        newAuthor.setName(newAuthorName);
 
        // Calling the add service
        addGraphicalRepresentation(context, newAuthor);
 
        // return newly created business object(s)
        return new Object[] { newAuthor };
	}

}
